import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:meta/meta.dart';

import '../data/repositories/auth_repository.dart';
import 'auth_event.dart';
import 'auth_state.dart';

class AuthBloc extends Bloc<AuthEvent, AuthState> {
  final AuthRepository authRepository;
  AuthBloc({@required this.authRepository}) : super(AuthState.UnInitialized);

  @override
  Stream<AuthState> mapEventToState(AuthEvent event) async* {
    switch (event) {
      case AuthEvent.AppStarted:
        yield AuthState.Loading;
        var hasAuthenticated = await authRepository.hasAuthenticated();
        if (hasAuthenticated)
          yield AuthState.Authenticated;
        else
          yield AuthState.UnAuthenticated;
        break;

      case AuthEvent.BeforeLogIn:
        yield AuthState.Loading;
        var response = await authRepository.authenticate();
        if (response.isSuccess)
          yield AuthState.Authenticated;
        else
          yield AuthState.UnAuthenticated;
        break;

      case AuthEvent.LoggedIn:
        yield AuthState.Authenticated;
        break;

      case AuthEvent.LoggedOut:
        yield AuthState.Loading;
        await authRepository.logOut();
        yield AuthState.UnAuthenticated;
        break;

      default:
        yield AuthState.UnInitialized;
        break;
    }
  }
}
