import 'package:dfapi_auth/models/dfapi_user_info.dart';
import 'package:dfapi_auth/presentation/pages/login_page.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:get_it/get_it.dart';

import 'dfapi_app_functions.dart';
import 'presentation/widgets/loading_widget.dart';
import 'bloc/auth_bloc.dart';
import 'bloc/auth_event.dart';
import 'bloc/auth_state.dart';
import 'data/repositories/auth_repository.dart';
import 'data/repository_contracts/auth_repository_contract.dart';
import 'models/dfapi_auth_request.dart';

class DfApiApp extends StatefulWidget {
  final DfApiAuthRequest request;

  static DfApiAppFunctions functions = DfApiAppFunctions(_DfApiApp.authBloc);
  static DfApiUserInfo userInfo;
  static String token;

  DfApiApp({
    Key key,
    this.request,
  }) : super(key: key);

  @override
  _DfApiApp createState() => _DfApiApp();
}

class _DfApiApp extends State<DfApiApp> {
  static AuthBloc authBloc;
  AuthRepository get authRepository =>
      AuthRepository(widget.request.configuration);

  _DfApiApp() : super() {
    var isRegistered = GetIt.instance.isRegistered<AuthRepositoryContract>();

    if (!isRegistered) {
      GetIt.instance
          .registerLazySingleton<AuthRepositoryContract>(() => authRepository);
    }
  }

  @override
  void initState() {
    DfApiApp.functions.getUserInfo().then((response) {
      if (response.isSuccess) DfApiApp.userInfo = response.value;
    });

    DfApiApp.functions.getToken().then((response) {
      if (response.isSuccess) DfApiApp.token = response.value;
    });

    authBloc = AuthBloc(authRepository: authRepository);
    authBloc.add(AuthEvent.AppStarted);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final request = widget.request;
    return Container(
      child: BlocProvider<AuthBloc>(
        create: (_) => authBloc,
        child: BlocBuilder<AuthBloc, AuthState>(
          builder: (BuildContext context, AuthState state) {
            switch (state) {
              case AuthState.Loading:
                return request.loadingWidget ?? LoadingWidget();
              case AuthState.UnInitialized:
              case AuthState.UnAuthenticated:
                return request.loginWidget ?? LoginPage();
              case AuthState.Authenticated:
                return request.child;
              default:
                return request.errorWidget ?? Text("ERROR!");
            }
          },
        ),
      ),
    );
  }
}
